//
//  SITOutdoorLocationOptions.h
//  SitumSDK
//
//  Created by Adrián Rodríguez on 24/01/2020.
//  Copyright © 2020 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
 Building detector strategy to find the building to start the indoor positioning. Default value is GPSProximity.
 */
typedef NS_ENUM(NSInteger, kSITBuildingDetector)
{
    /// Detect the nearest building using the GPS location distance to the building
    kSITGpsProximity = 0,
    /// Detect the building using BLE scans
    kSITBLE,
};

/**
 Accuracy you require for outdoor positions when the app is in background.
 */
typedef NS_ENUM(NSInteger, kSITBackgroundAccuracy)
{
    ///  Request a background outdoor accuracy of maximun accuracy
    kSITBackgroundAccuracyNavigation = 0,
    ///  Request a background outdoor accuracy of high accuracy
    kSITBackgroundAccuracyHigh = 1,
    ///  Request a background outdoor accuracy of medium accuracy
    kSITBackgroundAccuracyMedium = 2,
    ///  Request a background outdoor accuracy of low accuracy
    kSITBackgroundAccuracyLow = 3,

};

/**
 Outdoor location options used only in indoor-outdoor mode
*/
@interface SITOutdoorLocationOptions : NSObject <NSCopying>

/**
 Building detector strategy to find the building to start the indoor positioning
*/
@property (nonatomic) kSITBuildingDetector buildingDetector;

/**
 Controls whether outdoor positions should be used per the location engine.
 Default value: YES
 @Discussion: this flag is incompatible with the @link kSITBuildingDetector#kSITGpsProximity building detector option. If those parameters are requested the location engine will raise an error and stop. When this option is disabled (false value) outdoor locations won't be sent though location listener nor they will be uploaded to the realtime/historic dashboard.
 */
@property (nonatomic) BOOL enableOutdoorPositions;

/**
 Minimum accuracy required, in meters, for an outdoor position to be returned through the LocationManager delegate.
 Default value is zero, so all locations will be returned regardless of its accuracy.
 This parameter only accepts positive values. If a negative value is used, it will be ignored.
 */
@property (nonatomic) NSInteger minimumOutdoorLocationAccuracy;

/**
 Controls if the building selection stage should take into account building detection geofences. These building detection geofences are configured from the dashboard and must have a custom field with the key "trainer_metadata" and value "{ "passable":"True", "use_gps":"True"}".
 This mode is useful only for a `buildingDetector` of type kSITBLE.
 When activating this mode you should set `SITLocationRequest.useGps` to YES in the `SITLocationRequest` in order obtain proper locations inside these building detection geofences.
 */
@property (nonatomic) BOOL useGeofencesInBuildingSelector;

/**
  This property determines how fast the system computes a new outdoor location. It is defined in milliseconds.
 
  Possible values:
  - 0 means the system will compute outdoor locations as fast as possible.
  - >0 means the system will disable the sensors from update to update. (typical values are 5000, 10000, 30000 milliseconds).
 
 Default value: 0 millisecons
    
 Computing an outdoor location is a process that can take some seconds (typically between 5000 and 10000 milliseconds).
 You should expect a new outdoor location being computed in computeInterval + 5 to 10 seconds.
 
*/
@property (nonatomic) NSInteger computeInterval;

/**
 This property determines how fast the system returns outdoor locations. It is defined in milliseconds
 
 Possible values:
 - 0 means the system will output an outdoor location as soon as it is computed.
 - >0 means the system will output an outdoor location with the time interval defined in this property.
 
 Default value is 0
 
 See the property `computeInterval` to know how you can define the interval in which new outdoor positions are computed
*/
@property (nonatomic) NSInteger updateInterval;

/**
 This property determines the accuracy you require for outdoor positions when the app is in background. This property will be applied only if your app has any Background Location Update Mode enabled.
 
 Default value is kSITBackgroundAccuracyHigh
 */
@property (nonatomic) kSITBackgroundAccuracy backgroundAccuracy;

@end

